@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2007 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* Obstacle:
@@ -79,7 +79,7 @@ class Obstacle {
   /** This function calculates extents from pos, size, and rotation */
   void setExtents();
 
-    /** This function returns true if the obstacle is valid */
+  /** This function returns true if the obstacle is valid */
   virtual bool isValid() const;
 
   /** This function returns true if the obstacle has a flat top */
@@ -97,6 +97,9 @@ class Obstacle {
   /** This function prints the obstacle to the stream */
   virtual void print(std::ostream& out, const std::string& indent) const = 0;
 
+  /** This function prints the obstacle in Alias Wavefront format to the stream */
+  virtual void printOBJ(std::ostream&, const std::string&) const { return; }
+
   /** This function returns the position of this obstacle. */
   const Extents& getExtents() const;
 
@@ -187,6 +190,10 @@ class Obstacle {
       @c false if they can't. */
   bool isShootThrough() const;
 
+  /** This function returns @c true if tanks and bullets can pass through
+      this object, @c false if either can not */
+  bool isPassable() const;
+
   /** This function sets the "zFlip" flag of this obstacle, i.e. if it's
       upside down. */
   void setZFlip(void);
@@ -208,6 +215,9 @@ class Obstacle {
   bool isFromGroupDef() const;
   bool isFromContainer() const;
 
+  /** This function resets the object ID counter for printing OBJ files */
+  static void resetObjCounter();
+
   // inside sceneNodes
   void addInsideSceneNode(SceneNode* node);
   void freeInsideSceneNodeList();
@@ -224,6 +234,7 @@ class Obstacle {
   static const float maxExtent;
 
 
+
  protected:
   /** This function checks if a moving horizontal rectangle will hit a
       box-shaped obstacle, and if it does, computes the obstacle's normal
@@ -252,6 +263,10 @@ class Obstacle {
 		     float oWidth, float oBreadth, float oHeight,
 		     float* normal) const;
 
+  protected:
+    static int getObjCounter();
+    static void incObjCounter();
+
   protected:
     Extents extents;
     float pos[3];
@@ -266,6 +281,8 @@ class Obstacle {
     int insideNodeCount;
     SceneNode** insideNodes;
 
+  private:
+    static int objCounter;
 };
 
 //
@@ -277,41 +294,56 @@ inline const Extents& Obstacle::getExtents() const
   return extents;
 }
 
-inline const float*	Obstacle::getPosition() const
+inline const float* Obstacle::getPosition() const
 {
   return pos;
 }
 
-inline const float*	Obstacle::getSize() const
+inline const float* Obstacle::getSize() const
 {
   return size;
 }
 
-inline float		Obstacle::getRotation() const
+inline float Obstacle::getRotation() const
 {
   return angle;
 }
 
-inline float		Obstacle::getWidth() const
+inline float Obstacle::getWidth() const
 {
   return size[0];
 }
 
-inline float		Obstacle::getBreadth() const
+inline float Obstacle::getBreadth() const
 {
   return size[1];
 }
 
-inline float		Obstacle::getHeight() const
+inline float Obstacle::getHeight() const
 {
   return size[2];
 }
 
-inline void		Obstacle::get3DNormal(const float *p, float *n) const
+inline void Obstacle::get3DNormal(const float *p, float *n) const
 {
   getNormal(p, n);
 }
 
+inline bool Obstacle::isDriveThrough() const
+{
+  return driveThrough;
+}
+
+inline bool Obstacle::isShootThrough() const
+{
+  return shootThrough;
+}
+
+inline bool Obstacle::isPassable() const
+{
+  return (driveThrough && shootThrough);
+}
+
 inline void Obstacle::setSource(char _source)
 {
   source = _source;
@@ -338,6 +370,18 @@ inline bool Obstacle::isFromContainer() const
   return ((source & ContainerSource) != 0);
 }
 
+inline int Obstacle::getObjCounter()
+{
+  return objCounter;
+}
+inline void Obstacle::incObjCounter()
+{
+  objCounter++;
+}
+inline void Obstacle::resetObjCounter()
+{
+  objCounter = 0;
+}
 
 #endif // BZF_OBSTACLE_H
 
