@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2007 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,18 +7,19 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef _BITMAP_FONT_H_
 #define _BITMAP_FONT_H_
 
-#ifdef _MSC_VER
-  #pragma warning(disable : 4786)  // Disable warning message
-#endif
+/* common header */
+#include "common.h"
 
-#include "bzfgl.h"
+/* interface headers */
 #include "ImageFont.h"
+#include "bzfgl.h"
+#include "OpenGLGState.h"
 
 class BitmapFont : public ImageFont {
 public:
@@ -29,13 +30,23 @@ public:
   virtual bool isBuilt() const {return loaded;}
 
   virtual void filter(bool dofilter);
-  virtual void drawString(float scale, GLfloat color[3], const char *str, int len);
+  virtual void drawString(float scale, GLfloat color[4], const char *str, int len);
 
   virtual void free();
 
 private:
+  OpenGLGState gstate;
   unsigned char *bitmaps[MAX_TEXTURE_FONT_CHARS];
   bool	loaded;
 };
 
 #endif //_BITMAP_FONT_H_
+
+// Local Variables: ***
+// mode:C++ ***
+// tab-width: 8 ***
+// c-basic-offset: 2 ***
+// indent-tabs-mode: t ***
+// End: ***
+// ex: shiftwidth=2 tabstop=8
+
