@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2007 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
@@ -30,6 +30,7 @@
 #include "Team.h"
 #include "ServerList.h"
 #include "ErrorHandler.h"
+#include "cURLManager.h"
 
 StartupInfo startupInfo;
 
@@ -91,6 +92,7 @@ BZAdminClient::BZAdminClient(BZAdminUI* bzInterface)
 
   // set a default message mask
   showMessageType(MsgAddPlayer);
+  showMessageType(MsgAdminInfo);
   showMessageType(MsgKilled);
   showMessageType(MsgMessage);
   showMessageType(MsgNewRabbit);
@@ -231,7 +233,8 @@ BZAdminClient::ServerCode BZAdminClient::checkMessage() {
       players[p].isAdmin = false;
       if (ui != NULL)
 	ui->addedPlayer(p);
-      if (messageMask[MsgAddPlayer]) {
+	  // If you are an admin, then MsgAdminInfo will output the message
+      if (messageMask[MsgAddPlayer] && !players[getMyId()].isAdmin) {
 	Team temp;
 	std::string joinMsg = std::string("*** \'") + callsign + "\' joined the game as " +
 		temp.getName(players[p].team) + ".";
@@ -268,30 +271,44 @@ BZAdminClient::ServerCode BZAdminClient::checkMessage() {
       uint8_t numIPs;
       uint8_t tmp;
       vbuf = nboUnpackUByte(vbuf, numIPs);
-      for (i = 0; i < numIPs; ++i) {
+      if(numIPs > 1){
+	for (i = 0; i < numIPs; ++i) {
+	  vbuf = nboUnpackUByte(vbuf, tmp);
+	  vbuf = nboUnpackUByte(vbuf, p);
+	  vbuf = a.unpack(vbuf);
+	  players[p].ip = a.getDotNotation();
+	  if ((ui != NULL) && messageMask[MsgAdminInfo]){
+	    ui->outputMessage("*** IPINFO: " + players[p].name + " from "  +
+	      players[p].ip, Default);
+	  }
+	}
+      }
+      //Alternative to the MsgAddPlayer message
+      else if(numIPs == 1){
 	vbuf = nboUnpackUByte(vbuf, tmp);
 	vbuf = nboUnpackUByte(vbuf, p);
 	vbuf = a.unpack(vbuf);
 	players[p].ip = a.getDotNotation();
-      }
-      if (messageMask[MsgAdminInfo]) {
-	lastMessage.first = std::string("*** IP update received, ") +
-	  TextUtils::format("%d", numIPs) + " IP" + (numIPs == 1 ? "" : "s") +
-	  " updated.";
+	Team temp;
+	if (messageMask[MsgAdminInfo]){
+	  std::string joinMsg = std::string("*** \'") + players[p].name + "\' joined the game as " +
+	    temp.getName(players[p].team) + " from " + players[p].ip + ".";
+	  lastMessage.first = joinMsg;
+	}
       }
       break;
 
     case MsgScoreOver:
       if (messageMask[MsgScoreOver]) {
 	PlayerId id;
-	uint16_t team;
+	uint16_t _team;
 	vbuf = nboUnpackUByte(vbuf, id);
-	vbuf = nboUnpackUShort(vbuf, team);
+	vbuf = nboUnpackUShort(vbuf, _team);
 	it = players.find(id);
 	victimName = (it != players.end() ? it->second.name : "<unknown>");
-	if (team != (uint16_t)NoTeam) {
+	if (_team != (uint16_t)NoTeam) {
 	  Team temp;
-	  victimName = temp.getName((TeamColor)team);
+	  victimName = temp.getName((TeamColor)_team);
 	}
 	lastMessage.first = std::string("*** \'") + victimName + "\' won the game.";
       }
@@ -351,15 +368,15 @@ BZAdminClient::ServerCode BZAdminClient::checkMessage() {
       uint8_t numScores;
       vbuf = nboUnpackUByte(vbuf, numScores);
       for (i = 0; i < numScores; i++) {
-	uint16_t wins, losses, tks;
+	uint16_t winners, loosers, teamkillers;
 	vbuf = nboUnpackUByte(vbuf, p);
-	vbuf = nboUnpackUShort(vbuf, wins);
-	vbuf = nboUnpackUShort(vbuf, losses);
-	vbuf = nboUnpackUShort(vbuf, tks);
+	vbuf = nboUnpackUShort(vbuf, winners);
+	vbuf = nboUnpackUShort(vbuf, loosers);
+	vbuf = nboUnpackUShort(vbuf, teamkillers);
 	if ((iter = players.find(p)) != players.end()) {
-	  iter->second.wins = wins;
-	  iter->second.losses = losses;
-	  iter->second.tks = tks;
+	  iter->second.wins   = winners;
+	  iter->second.losses = loosers;
+	  iter->second.tks    = teamkillers;
 	}
       }
       if (messageMask[MsgScore]) {
@@ -384,10 +401,10 @@ BZAdminClient::ServerCode BZAdminClient::checkMessage() {
 			   TeamColor(250 - dst) : NoTeam);
       if (messageMask[MsgMessage]) {
 	lastMessage.first = formatMessage((char*)vbuf, src, dst,dstTeam, me);
-	PlayerIdMap::const_iterator iter = players.find(src);
-	lastMessage.second = (iter == players.end() ?
+	PlayerIdMap::const_iterator iterator = players.find(src);
+	lastMessage.second = (iterator == players.end() ?
 			      colorMap[NoTeam] :
-			      colorMap[iter->second.team]);
+			      colorMap[iterator->second.team]);
       }
       break;
     }
@@ -440,7 +457,8 @@ void BZAdminClient::outputServerList() const {
       }
     }
     serverList.checkEchos(&startupInfo);
-    TimeKeeper::sleep(1.0f);
+    cURLManager::perform();
+    TimeKeeper::sleep(1.0);
   }
   // what is your final answer?
   serverList.checkEchos(&startupInfo);
@@ -549,21 +567,36 @@ std::string BZAdminClient::formatMessage(const std::string& msg, PlayerId src,
   const std::string dstName = (players.count(dst) ? players[dst].name :
 			       "(UNKNOWN)");
 
+  // display action messages differently
+  bool isAction = false;
+  std::string message;
+  if ((msg[0] == '*') && (msg[1] == ' ') &&
+      (msg[msg.size () - 1] == '*') && (msg[msg.size () - 2] == '\t')) {
+    isAction = true;
+    message = msg.substr(2, msg.size() - 4);
+  } else {
+    message = msg;
+  }
+
   // direct message to or from me
   if (dst == me || players.count(dst)) {
     if (!(src == me && dst == me)) {
-      formatted += "[";
       if (src == me) {
-	formatted += "->";
-	formatted += dstName;
+	if (isAction) {
+	  formatted += "[->" + message + "]";
+	} else {
+	  formatted += "[->" + dstName + "] " + message;
+	}
+      } else {
+	if (isAction) {
+	  formatted += "[" + message + "->]";
+	} else {
+	  formatted += "[" + srcName + "->] " + message;
+	}
       }
-      else {
-	formatted += srcName;
-	formatted += "->";
-      }
-      formatted += "] ";
+    } else {
+      formatted += message;
     }
-    formatted += msg;
   }
 
   // public or admin or team message
@@ -572,9 +605,12 @@ std::string BZAdminClient::formatMessage(const std::string& msg, PlayerId src,
       formatted += "[Admin] ";
     else if (dstTeam != NoTeam)
       formatted += "[Team] ";
-    formatted += srcName;
-    formatted += ": ";
-    formatted += msg;
+
+    if (!isAction) {
+      formatted += srcName;
+      formatted += ": ";
+    }
+    formatted += message;
   }
 
   return formatted;
@@ -596,12 +632,13 @@ void BZAdminClient::waitForServer() {
   if (sLink.getState() == ServerLink::Okay) {
     sendMessage("bzadminping", me);
     std::string expected = formatMessage("bzadminping", me, me, NoTeam, me);
+    std::string noTalk = formatMessage("We're sorry, you are not allowed to talk!", ServerPlayer, me, NoTeam, me);
     std::string str;
     BZAdminUI* tmpUI = ui;
     ui = NULL;
     do {
       checkMessage();
-    } while (lastMessage.first != expected);
+    } while (lastMessage.first != expected && lastMessage.first != noTalk);
     ui = tmpUI;
   }
   messageMask[MsgMessage] = tmp;
