@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2007 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,31 +7,28 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
 #include "HUDRenderer.h"
 
-/* system implementation headers */
+// system headers
 #include <time.h>
-#include <math.h>
-#include <string>
 
 /* common implementation headers */
-#include "StateDatabase.h"
 #include "BundleMgr.h"
 #include "Bundle.h"
-#include "Team.h"
 #include "FontManager.h"
 #include "BZDBCache.h"
 
 /* local implementation headers */
 #include "LocalPlayer.h"
 #include "World.h"
-#include "RemotePlayer.h"
 #include "HUDui.h"
-#include "HUDuiTypeIn.h"
+#include "Roaming.h"
+#include "playing.h"
+#include "TextUtils.h"
 
 
 //
@@ -43,12 +40,6 @@
 const float		HUDRenderer::altitudeOffset = 20.0f;
 const GLfloat		HUDRenderer::black[3] = { 0.0f, 0.0f, 0.0f };
 std::string		HUDRenderer::headingLabel[36];
-std::string		HUDRenderer::scoreSpacingLabel("88% 888 (888-888)[88]");
-std::string		HUDRenderer::scoreLabel("Score");
-std::string		HUDRenderer::killLabel("Kills");
-std::string		HUDRenderer::teamScoreSpacingLabel("888 (888-888) 888");
-std::string		HUDRenderer::teamScoreLabel("Team Score");
-std::string		HUDRenderer::playerLabel("Player");
 std::string		HUDRenderer::restartLabelFormat("Press %s to start");
 std::string		HUDRenderer::resumeLabel("Press Pause to resume");
 std::string		HUDRenderer::autoPilotLabel("AutoPilot on");
@@ -73,16 +64,12 @@ HUDRenderer::HUDRenderer(const BzfDisplay* _display,
 				fps(-1.0),
 				drawTime(-1.0),
 				restartLabel(restartLabelFormat),
-				roamingLabel("observing"),
 				showCompose(false),
 				showCracks(true),
-				huntIndicator(false),
-				hunting(false),
-				huntPosition(0),
-				huntSelection(false),
-				showHunt(false),
 				dater(false),
-				lastTimeChange(time(NULL))
+				lastTimeChange(time(NULL)),
+				triangleCount(0),
+				radarTriangleCount(0)
 {
   if (BZDB.eval("timedate") == 0) //we just want the time
     dater = false;
@@ -123,6 +110,9 @@ HUDRenderer::HUDRenderer(const BzfDisplay* _display,
   composeTypeIn->setMaxLength(MessageLen - 1);
   composeTypeIn->showFocus(false);
 
+  // create scoreboard renderer
+  scoreboard = new ScoreboardRenderer();
+
   // initialize fonts
   resize(true);
 }
@@ -136,6 +126,14 @@ HUDRenderer::~HUDRenderer()
   delete composeTypeIn;
 }
 
+
+ScoreboardRenderer *HUDRenderer::getScoreboard ()
+{
+  return scoreboard;
+}
+
+
+
 void			HUDRenderer::resizeCallback(void* self)
 {
   ((HUDRenderer*)self)->resize(false);
@@ -169,6 +167,11 @@ void			HUDRenderer::resize(bool firstTime)
   setHeadingFontSize(w, vh);
   setComposeFontSize(w, vh);
   setLabelsFontSize(w, vh);
+
+  // set scoreboard window size & location
+  const float sby = window.getViewHeight() - majorFontHeight - alertFontHeight * 2.0f;
+  scoreboard->setWindowSize (0.01f * window.getWidth(), sby,
+	0.98f * window.getWidth(), sby);
 }
 
 int			HUDRenderer::getNoMotionSize() const
@@ -201,6 +204,7 @@ void			HUDRenderer::setAlertFontSize(int, int height)
   FontManager &fm = FontManager::instance();
   alertFontFace = fm.getFaceID(BZDB.get("sansSerifFont"));
   alertFontSize = floorf(s);
+  alertFontHeight = fm.getStrHeight(alertFontFace, alertFontSize, " ");
 
   for (int i = 0; i < MaxAlerts; i++)
     if (alertClock[i].isOn())
@@ -213,6 +217,7 @@ void			HUDRenderer::setMajorFontSize(int, int height)
   FontManager &fm = FontManager::instance();
   majorFontFace = fm.getFaceID(BZDB.get("serifFont"));
   majorFontSize = floorf(s);
+  majorFontHeight = fm.getStrHeight(majorFontFace, majorFontSize, " ");
 }
 
 void			HUDRenderer::setMinorFontSize(int, int height)
@@ -239,17 +244,6 @@ void			HUDRenderer::setMinorFontSize(int, int height)
     minorFontSize = 16;
     break;
   }
-
-  huntArrowWidth = fm.getStrLength(minorFontFace, minorFontSize, "-> ");
-  huntedArrowWidth = fm.getStrLength(minorFontFace, minorFontSize, "Hunt->");
-
-  scoreLabelWidth = fm.getStrLength(minorFontFace, minorFontSize, scoreSpacingLabel);
-  killsLabelWidth = fm.getStrLength(minorFontFace, minorFontSize, killLabel);
-  teamScoreLabelWidth = fm.getStrLength(minorFontFace, minorFontSize, teamScoreSpacingLabel);
-
-  const float spacing = fm.getStrLength(minorFontFace, minorFontSize, " ");
-  scoreLabelWidth += spacing;
-  killsLabelWidth += spacing;
 }
 
 void			HUDRenderer::setHeadingFontSize(int, int height)
@@ -305,6 +299,7 @@ void			HUDRenderer::setRoaming(bool _roaming)
 void			HUDRenderer::setDim(bool _dim)
 {
   dim = _dim;
+  scoreboard->setDim (_dim);
 }
 
 void			HUDRenderer::setPlayerHasHighScore(bool hasHigh)
@@ -344,6 +339,16 @@ void			HUDRenderer::setDrawTime(float drawTimeInseconds)
   drawTime = drawTimeInseconds;
 }
 
+void			HUDRenderer::setFrameTriangleCount(int tpf)
+{
+  triangleCount = tpf;
+}
+
+void			HUDRenderer::setFrameRadarTriangleCount(int rtpf)
+{
+  radarTriangleCount = rtpf;
+}
+
 void			HUDRenderer::setAlert(int index, const char* string,
 						float duration, bool warning)
 {
@@ -453,7 +458,7 @@ void			HUDRenderer::setCracks(bool _showCracks)
 {
   if ((showCracks != _showCracks) && _showCracks) {
     initCracks();
-    crackStartTime = TimeKeeper::getCurrent();
+    crackStartTime = TimeKeeper::getTick();
   }
   showCracks = _showCracks;
 }
@@ -479,11 +484,6 @@ void			HUDRenderer::setRestartKeyLabel(const std::string& label)
   restartLabelWidth = fm.getStrLength(bigFontFace, bigFontSize, restartLabel);
 }
 
-void			HUDRenderer::setRoamingLabel(const std::string& label)
-{
-  roamingLabel = label;
-}
-
 void			HUDRenderer::setTimeLeft(uint32_t _timeLeft)
 {
   timeLeft = _timeLeft;
@@ -620,12 +620,51 @@ void			HUDRenderer::render(SceneRenderer& renderer)
   }
 
   OpenGLGState::resetState();
-  if (playing)
-    renderPlaying(renderer);
-  else if (roaming)
-    renderRoaming(renderer);
-  else
-    renderNotPlaying(renderer);
+  if (!BZDB.isTrue("noGUI")) {
+    if (playing) {
+      renderPlaying(renderer);
+    }
+    else if (roaming) {
+      renderRoaming(renderer);
+    }
+    else {
+      renderNotPlaying(renderer);
+    }
+  }
+  else {
+    const bool showTimes = (fps > 0.0f) || (drawTime > 0.0f) ||
+			   (triangleCount > 0) || (radarTriangleCount > 0);
+    const bool showTankLabels = BZDB.isTrue("displayLabels");
+
+    if (showCompose || showTimes || showTankLabels) {
+      // get view metrics
+      const int width = window.getWidth();
+      const int height = window.getHeight();
+      const int viewHeight = window.getViewHeight();
+      const int ox = window.getOriginX();
+      const int oy = window.getOriginY();
+      // use one-to-one pixel projection
+      glScissor(ox, oy + height - viewHeight, width, viewHeight);
+      glMatrixMode(GL_PROJECTION);
+      glLoadIdentity();
+      glOrtho(0.0, width, viewHeight - height, viewHeight, -1.0, 1.0);
+      glMatrixMode(GL_MODELVIEW);
+      glPushMatrix();
+      glLoadIdentity();
+
+      if (showCompose) {
+	renderCompose(renderer);
+      }
+      if (showTimes) {
+	renderTimes();
+      }
+      if (showTankLabels) {
+	renderTankLabels(renderer);
+      }
+
+      glPopMatrix();
+    }
+  }
 }
 
 void			HUDRenderer::renderAlerts(void)
@@ -641,8 +680,14 @@ void			HUDRenderer::renderAlerts(void)
   for (int i = 0; i < MaxAlerts; i++) {
     if (alertClock[i].isOn()) {
       hudColor3fv(alertColor[i]);
+      std::string newAlertLabel = (dim ? ColorStrings[DimColor] : "") + alertLabel[i];
+      // FIXME: this assumes that there's not more than one reset in the string.
+      if (dim) {
+	newAlertLabel.insert(newAlertLabel.find(ColorStrings[ResetColor], 0) - 1
+			     + ColorStrings[ResetColor].size(), ColorStrings[DimColor]);
+      }
       fm.drawString(centerx - 0.5f * alertLabelWidth[i], y, 0,
-		    alertFontFace, alertFontSize, alertLabel[i]);
+		    alertFontFace, alertFontSize, newAlertLabel);
       y -= fm.getStrHeight(alertFontFace, alertFontSize, " ");
     }
   }
@@ -650,24 +695,24 @@ void			HUDRenderer::renderAlerts(void)
 
 void			HUDRenderer::renderStatus(void)
 {
-  LocalPlayer* player = LocalPlayer::getMyTank();
-  if (!player || !World::getWorld()) return;
+  LocalPlayer* myTank = LocalPlayer::getMyTank();
+  if (!myTank || !World::getWorld()) return;
 
   Bundle *bdl = BundleMgr::getCurrentBundle();
 
   FontManager &fm = FontManager::instance();
 
-  char buffer[60];
+  char buffer[80];
   const float h = fm.getStrHeight(majorFontFace, majorFontSize, " ");
   float x = 0.25f * h;
   float y = (float)window.getViewHeight() - h;
-  TeamColor teamIndex = player->getTeam();
-  FlagType* flag = player->getFlag();
+  TeamColor teamIndex = myTank->getTeam();
+  FlagType* flag = myTank->getFlag();
 
   // print player name and score in upper left corner in team (radar) color
   if (!roaming && (!playerHasHighScore || scoreClock.isOn())) {
-    sprintf(buffer, "%s: %d", player->getCallSign(), player->getScore());
-    hudColor3fv(Team::getRadarColor(teamIndex));
+    sprintf(buffer, "%s: %d", myTank->getCallSign(), myTank->getScore());
+    hudColor3fv(Team::getRadarColor(teamIndex, World::getWorld()->allowRabbit()));
     fm.drawString(x, y, 0, majorFontFace, majorFontSize, buffer);
   }
 
@@ -701,12 +746,110 @@ void			HUDRenderer::renderStatus(void)
       sprintf(buffer, "%4d.%02d.%02d", 1900 + userTime.tm_year, userTime.tm_mon + 1, userTime.tm_mday);
     else
       sprintf(buffer, "%2d:%2.2d", userTime.tm_hour, userTime.tm_min);
-
     x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(majorFontFace, majorFontSize, buffer);
     hudColor3fv(messageColor);
     fm.drawString(x, y, 0, majorFontFace, majorFontSize, buffer);
   }
 
+  // print current position of tank
+  if (BZDB.isTrue("showCoordinates") && !(roaming && BZDB.isTrue("showVelocities")) ) {
+    y -= float(1.0*h);
+    sprintf(buffer, "[%d %d %d]", (int)myTank->getPosition()[0],
+	    (int)myTank->getPosition()[1], (int)myTank->getPosition()[2]);
+    x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(majorFontFace, majorFontSize, buffer);
+    fm.drawString(x, y, 0, majorFontFace, majorFontSize, buffer);
+    y += float(1.5*h);
+  }
+
+  if (roaming && BZDB.isTrue("showVelocities"))
+  {
+    Player *target = ROAM.getTargetTank();
+    if (target) {
+      float vel[3] = {0};
+      memcpy(vel,target->getVelocity(),sizeof(float)*3);
+  
+      float apparentVel[3] = {0};
+      memcpy(apparentVel,target->getApparentVelocity(),sizeof(float)*3);
+  
+      float linSpeed = sqrt(vel[0]*vel[0]+vel[1]*vel[1]);
+      float vertSpeed = vel[2];
+      float rotSpeed = fabs(target->getAngularVelocity());
+      float apparentLinSpeed = sqrt(apparentVel[0]*apparentVel[0]+apparentVel[1]*apparentVel[1]);
+
+      // calc maximum apparent velocity value for each 0.5s interval
+      static float maxApparentLinSpeed = 0.0f;
+      static TimeKeeper maxApparentLinTime = TimeKeeper::getStartTime();
+
+      if (maxApparentLinSpeed > apparentLinSpeed &&
+          TimeKeeper::getTick() - maxApparentLinTime < 0.5f) {
+        apparentLinSpeed = maxApparentLinSpeed;
+      }
+      else {
+        maxApparentLinSpeed = apparentLinSpeed;
+        maxApparentLinTime = TimeKeeper::getTick();
+      }
+  
+      float smallZHeight = fm.getStrHeight(minorFontFace,minorFontSize,std::string("X"))*1.125f;
+      float drawY = y - smallZHeight;
+      // draw header
+      x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize, "Target Info");
+      fm.drawString(x, drawY, 0, minorFontFace, minorFontSize, "Target Info");
+
+      // draw the postion
+      sprintf(buffer, "Position [%d %d %d]", (int)myTank->getPosition()[0], (int)myTank->getPosition()[1], (int)myTank->getPosition()[2]);
+      x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
+      fm.drawString(x,drawY-smallZHeight, 0, minorFontFace, minorFontSize, buffer);
+
+      std::string label = "Linear Speed:";
+      if ( linSpeed > target->getMaxSpeed() )
+        label += "!";
+  
+      sprintf(buffer,"%s%5.2f",label.c_str(),linSpeed);
+      if (BZDB.evalInt("showVelocities") > 1)
+        sprintf(buffer,"%s%5.2f(%5.2f)",label.c_str(),linSpeed,apparentLinSpeed);
+  
+      x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
+      fm.drawString(x,drawY-smallZHeight*2.0f, 0, minorFontFace, minorFontSize, buffer);
+  
+      sprintf(buffer,"Vertical Speed:%5.2f",vertSpeed);
+      if (BZDB.evalInt("showVelocities") > 1)
+        sprintf(buffer,"Vertical Speed:%5.2f(%5.2f)",vertSpeed,apparentVel[2]);
+  
+      x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
+      fm.drawString(x, drawY-smallZHeight*3.0f, 0, minorFontFace, minorFontSize, buffer);
+  
+      label = "Angular Speed:";
+      if (rotSpeed > BZDB.eval(StateDatabase::BZDB_TANKANGVEL))
+        label += "!";
+      sprintf(buffer,"%s%5.2f",label.c_str(),rotSpeed);
+      x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
+      fm.drawString(x,drawY-smallZHeight*4.0f, 0, minorFontFace, minorFontSize, buffer);
+  
+      float shotTime = (float)target->getShotStatistics()->getLastShotTimeDelta();
+      float shotDeviation = (float)target->getShotStatistics()->getLastShotDeviation();
+  
+      sprintf(buffer,"Last Shot Info  Time:%6.4f  Deviation:%6.3f", shotTime, shotDeviation);
+      x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
+      fm.drawString(x,drawY-smallZHeight*5.0f, 0, minorFontFace, minorFontSize, buffer);
+  
+      float offset = 6.0f;
+      if (BZDB.evalInt("showVelocities") > 2) {
+        offset += 1.0f;
+        sprintf(buffer,"ReportedHits %d ComputedHits %d ratio %f", target->reportedHits, target->computedHits,(float)target->reportedHits/(float)target->computedHits);
+        x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
+        fm.drawString(x,drawY-smallZHeight*offset, 0, minorFontFace, minorFontSize, buffer);
+      }
+  
+      offset += 1.5f;
+      scoreboard->setTeamScoreY(drawY - smallZHeight*offset);
+    }
+    else
+      scoreboard->setTeamScoreY(0);
+  }
+  else
+    scoreboard->setTeamScoreY(0);
+
+
   // print status top-center
   static const GLfloat redColor[3] = { 1.0f, 0.0f, 0.0f };
   static const GLfloat yellowColor[3] = { 1.0f, 1.0f, 0.0f };
@@ -727,7 +870,7 @@ void			HUDRenderer::renderStatus(void)
       sprintf(buffer, "0:%02d   ", t);
   }
   if (!roaming) {
-    switch (player->getFiringStatus()) {
+    switch (myTank->getFiringStatus()) {
       case LocalPlayer::Deceased:
 	strcat(buffer, bdl->getLocalString("Dead").c_str());
 	break;
@@ -737,7 +880,7 @@ void			HUDRenderer::renderStatus(void)
 	    World::getWorld()->allowShakeTimeout()) {
 	  /* have a bad flag -- show time left 'til we shake it */
 	  statusColor = yellowColor;
-	  sprintf(buffer, bdl->getLocalString("%.1f").c_str(), player->getFlagShakingTime());
+	  sprintf(buffer, bdl->getLocalString("%.1f").c_str(), myTank->getFlagShakingTime());
 	} else {
 	  statusColor = greenColor;
 	  strcat(buffer, bdl->getLocalString("Ready").c_str());
@@ -745,8 +888,11 @@ void			HUDRenderer::renderStatus(void)
 	break;
 
       case LocalPlayer::Loading:
-	statusColor = redColor;
-	sprintf(buffer, bdl->getLocalString("Reloaded in %.1f").c_str(), player->getReloadTime());
+
+    if (World::getWorld()->getMaxShots() != 0) {
+	  statusColor = redColor;
+	  sprintf(buffer, bdl->getLocalString("Reloaded in %.1f").c_str(), myTank->getReloadTime());
+	}
 	break;
 
       case LocalPlayer::Sealed:
@@ -761,7 +907,8 @@ void			HUDRenderer::renderStatus(void)
 
   if (roaming) {
     statusColor = messageColor;
-    strcat(buffer, roamingLabel.c_str());
+    if (dim) strcat(buffer, ColorStrings[DimColor].c_str());
+    strcat(buffer, ROAM.getRoamingLabel().c_str());
   }
 
   x = 0.5f * ((float)window.getWidth() - fm.getStrLength(majorFontFace, majorFontSize, buffer));
@@ -781,209 +928,21 @@ int HUDRenderer::tankScoreCompare(const void* _a, const void* _b)
 
 int HUDRenderer::teamScoreCompare(const void* _c, const void* _d)
 {
-
   Team* c = World::getWorld()->getTeams()+*(int*)_c;
   Team* d = World::getWorld()->getTeams()+*(int*)_d;
 
   return (d->won-d->lost) - (c->won-c->lost);
 }
 
-void		HUDRenderer::setHuntPosition(int _huntPosition)
-{
-  huntPosition = _huntPosition;
-}
-
-int			HUDRenderer::getHuntPosition() const
-{
-  return huntPosition;
-}
-
-bool			HUDRenderer::getHuntSelection() const
-{
-  return huntSelection;
-}
-
-void			HUDRenderer::setHuntSelection(bool _huntSelection)
-{
-  huntSelection = _huntSelection;
-}
-
-void		HUDRenderer::setHuntIndicator(bool _huntIndicator)
-{
-  huntIndicator = _huntIndicator;
-}
-
-bool			HUDRenderer::getHuntIndicator() const
-{
-  return huntIndicator;
-}
-
-void		HUDRenderer::setHunting(bool _hunting)
-{
-  hunting = _hunting;
-}
-
-bool			HUDRenderer::getHunting() const
-{
-  return hunting;
-}
-
-bool			HUDRenderer::getHunt() const
-{
-  return showHunt;
-}
-
-void			HUDRenderer::setHunt(bool _showHunt)
-{
-  showHunt = _showHunt;
-}
-
-void			HUDRenderer::renderScoreboard(void)
-{
-  int i, j;
-  bool huntPlayerAlive = false;
-
-  LocalPlayer* myTank = LocalPlayer::getMyTank();
-  if (!myTank || !World::getWorld()) return;
-
-  Bundle *bdl = BundleMgr::getCurrentBundle();
-  FontManager &fm = FontManager::instance();
-
-  const float x1 = 0.01f * window.getWidth();
-  const float x2 = x1 + scoreLabelWidth;
-  const float x3 = x2 + scoreLabelWidth;
-  const float x5 = (1.0f - 0.01f) * window.getWidth() - teamScoreLabelWidth;
-  const float y0 = (float)window.getViewHeight() - fm.getStrHeight(majorFontFace, majorFontSize, " ")
-      - fm.getStrHeight(alertFontFace, alertFontSize, " ") * 2.0f;
-  hudColor3fv(messageColor);
-  fm.drawString(x1, y0, 0, minorFontFace, minorFontSize, bdl->getLocalString(scoreLabel));
-  fm.drawString(x2, y0, 0, minorFontFace, minorFontSize, bdl->getLocalString(killLabel));
-  fm.drawString(x3, y0, 0, minorFontFace, minorFontSize, bdl->getLocalString(playerLabel));
-  if (!World::getWorld()->allowRabbit())
-    fm.drawString(x5, y0, 0, minorFontFace, minorFontSize, bdl->getLocalString(teamScoreLabel));
-  const float dy = fm.getStrHeight(minorFontFace, minorFontSize, " ");
-  int y = (int)(y0 - dy);
-
-  // make room for the status marker
-  const float xs = x3 - fm.getStrLength(minorFontFace, minorFontSize, "+|");
-
-  // print non-observing players sorted by score, print observers last
-  int plrCount = 0;
-  int obsCount = 0;
-  const int curMaxPlayers = World::getWorld()->getCurMaxPlayers();
-  int* players = new int[curMaxPlayers];
-  RemotePlayer* rp;
-
-  for (j = 0; j < curMaxPlayers; j++) {
-    if ((rp = World::getWorld()->getPlayer(j))) {
-      if (rp->getTeam() != ObserverTeam)
-	players[plrCount++] = j;
-      else
-	players[curMaxPlayers - (++obsCount)] = j;
-    }
-  }
-
-  qsort(players, plrCount, sizeof(int), tankScoreCompare);
-
-  // list player scores
-  bool drewMyScore = false;
-  for (i = 0; i < plrCount; i++) {
-    RemotePlayer* player = World::getWorld()->getPlayer(players[i]);
-    if(getHunt()) {
-      // Make the selection marker wrap.
-      if(getHuntPosition() >= plrCount) setHuntPosition(0);
-      if(getHuntPosition() < 0) setHuntPosition(plrCount-1);
-
-      // toggle the hunt indicator if this is the current player pointed to
-      if(getHuntPosition() == i) {
-	setHuntIndicator(true);
-	// If hunt is selected set this player to be hunted
-	if(getHuntSelection()) {
-	  player->setHunted(true);
-	  setHunting(true);
-	  setHuntSelection(false);
-	  setHunt(false);
-	  huntPlayerAlive = true; // hunted player is alive since you selected him
-	}
-      } else {
-	setHuntIndicator(false);
-      }
-    } else {
-      setHuntIndicator(false);
-      if (!getHunting()) player->setHunted(false); // if not hunting make sure player isn't hunted
-      else if (player->isHunted()) huntPlayerAlive = true; // confirm hunted player is alive
-    }
-    bool myTurn = false;
-    if (!drewMyScore && myTank->getTeam() != ObserverTeam)
-      if (World::getWorld()->allowRabbit()) {
-	myTurn = myTank->getRabbitScore() > player->getRabbitScore();
-      } else {
-	myTurn = myTank->getScore() > player->getScore();
-      }
-    if (myTurn) {
-      setHuntIndicator(false); // don't hunt myself
-      // if i have greater score than remote player draw my name here
-      drawPlayerScore(myTank, x1, x2, x3, xs, (float)y);
-      drewMyScore = true;
-      y -= (int)dy;
-    }
-    if(getHunt() && getHuntPosition() == i) setHuntIndicator(true);// set hunt indicator back to normal
-    drawPlayerScore(player, x1, x2, x3, xs, (float)y);//then draw the remote player
-    y -= (int)dy;
-  }
-  if (!huntPlayerAlive && getHunting()) setHunting(false); //stop hunting if hunted player is dead
-  if (!drewMyScore && (myTank->getTeam() != ObserverTeam)) {
-    setHuntIndicator(false); // don't hunt myself
-    // if my score is smaller or equal to last remote player draw my score here
-    drawPlayerScore(myTank, x1, x2, x3, xs, (float)y);
-    y -= (int)dy;
-    drewMyScore = true;
-  }
-
-  // list observers
-  y -= (int)dy;
-  for (i = curMaxPlayers - 1; i >= curMaxPlayers - obsCount; --i) {
-    RemotePlayer* player = World::getWorld()->getPlayer(players[i]);
-    drawPlayerScore(player, x1, x2, x3, xs, (float)y);
-    y -= (int)dy;
-  }
-  if (!drewMyScore) {
-    // if I am an observer, list my name
-    drawPlayerScore(myTank, x1, x2, x3, xs, (float)y);
-  }
-
-  delete[] players;
-
-  // print teams sorted by score
-  int teams[NumTeams];
-  int teamCount = 0;
-
-  y = (int)y0;
-  for (i = RedTeam; i < NumTeams; i++) {
-    if (!Team::isColorTeam(TeamColor(i))) continue;
-    const Team* team = World::getWorld()->getTeams() + i;
-    if (team->size == 0) continue;
-    teams[teamCount++] = i;
-  }
-
-  qsort(teams, teamCount, sizeof(int), teamScoreCompare);
-
-  y -= (int)dy;
-  for (i = 0 ; i < teamCount; i++){
-    drawTeamScore(teams[i], x5, (float)y);
-    y -= (int)dy;
-  }
-}
 
 void			HUDRenderer::renderTankLabels(SceneRenderer& renderer)
 {
   if (!World::getWorld()) return;
 
   int offset = window.getViewHeight() - window.getHeight();
-  const int curMaxPlayers = World::getWorld()->getCurMaxPlayers();
 
-  GLint view[]={window.getOriginX(), window.getOriginY(),
-		window.getWidth(), window.getHeight()};
+  GLint view[] = {window.getOriginX(), window.getOriginY(),
+		 window.getWidth(), window.getHeight()};
   const GLfloat *projf = renderer.getViewFrustum().getProjectionMatrix();
   const GLfloat *modelf = renderer.getViewFrustum().getViewMatrix();
 
@@ -999,9 +958,9 @@ void			HUDRenderer::renderTankLabels(SceneRenderer& renderer)
     if (pl && pl->isAlive()) {
       const char *name = pl->getCallSign();
       double x, y, z;
-      hudSColor3fv(Team::getRadarColor(pl->getTeam()));
+      hudSColor3fv(Team::getRadarColor(pl->getTeam(), World::getWorld()->allowRabbit()));
       gluProject(pl->getPosition()[0], pl->getPosition()[1],
-		 pl->getPosition()[2], model, proj, view, &x, &y, &z);
+		 pl->getPosition()[2]/*+BZDB.eval(StateDatabase::BZDB_MUZZLEHEIGHT)*3.0f*/, model, proj, view, &x, &y, &z);
       if (z >= 0.0 && z <= 1.0) {
 	FontManager &fm = FontManager::instance();
 	fm.drawString(float(x) - fm.getStrLength(labelsFontFace, labelsFontSize, name) / 2.0f,
@@ -1018,14 +977,25 @@ void			HUDRenderer::renderTankLabels(SceneRenderer& renderer)
 			(2.0f * fm.getStrHeight(labelsFontFace, labelsFontSize, fname)),
 			0, labelsFontFace, labelsFontSize, fname);
 	}
-      }
+
+		if (roaming && BZDB.isTrue("showVelocities")) 
+		{
+			float vel[3] = {0};
+			memcpy(vel,pl->getVelocity(),sizeof(float)*3);
+			std::string speedStr = TextUtils::format("[%5.2f]",sqrt(vel[0]*vel[0]+vel[1]*vel[1]));
+			fm.drawString(float(x) - fm.getStrLength(labelsFontFace, labelsFontSize, speedStr.c_str()) / 2.0f,
+				float(y) + offset -
+				(3.0f * fm.getStrHeight(labelsFontFace, labelsFontSize, speedStr.c_str())),
+				0, labelsFontFace, labelsFontSize, speedStr.c_str());
+		}
+	  }
     }
   }
 }
 
 void			HUDRenderer::renderCracks()
 {
-  double delta = (TimeKeeper::getCurrent() - crackStartTime) * 5.0;
+  double delta = (TimeKeeper::getTick() - crackStartTime) * 5.0;
   if (delta > 1.0)
     delta = 1.0;
   int maxLevels = (int) (HUDCrackLevels * delta);
@@ -1075,6 +1045,24 @@ void			HUDRenderer::renderTimes(void)
 		3.0f * fm.getStrHeight(headingFontFace, headingFontSize, "0"), 0,
 		headingFontFace, headingFontSize, buf);
   }
+  float triCountYOffset = 4.5f;
+  if (radarTriangleCount > 0) {
+    char buf[20];
+    sprintf(buf, "rtris: %i", radarTriangleCount);
+    hudColor3f(1.0f, 1.0f, 1.0f);
+    fm.drawString((float)(centerx - maxMotionSize), (float)centery + (float)maxMotionSize +
+		triCountYOffset * fm.getStrHeight(headingFontFace, headingFontSize, "0"), 0,
+		headingFontFace, headingFontSize, buf);
+    triCountYOffset += 1.5f;
+  }
+  if (triangleCount > 0) {
+    char buf[20];
+    sprintf(buf, "tris: %i", triangleCount);
+    hudColor3f(1.0f, 1.0f, 1.0f);
+    fm.drawString((float)(centerx - maxMotionSize), (float)centery + (float)maxMotionSize +
+		triCountYOffset * fm.getStrHeight(headingFontFace, headingFontSize, "0"), 0,
+		headingFontFace, headingFontSize, buf);
+  }
   if (drawTime > 0.0f) {
     char buf[20];
     sprintf(buf, "time: %dms", (int)(drawTime * 1000.0f));
@@ -1334,7 +1322,6 @@ void			HUDRenderer::renderPlaying(SceneRenderer& renderer)
   FontManager &fm = FontManager::instance();
 
   // use one-to-one pixel projection
-
   glScissor(ox, oy + height - viewHeight, width, viewHeight);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -1343,12 +1330,17 @@ void			HUDRenderer::renderPlaying(SceneRenderer& renderer)
   glPushMatrix();
   glLoadIdentity();
 
-  LocalPlayer *myTank = LocalPlayer::getMyTank();
+  // cover the lower portion of the screen when burrowed
+  const LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank && myTank->getPosition()[2] < 0.0f) {
     glColor4f(0.02f, 0.01f, 0.01f, 1.0);
     glRectf(0, 0, (float)width, (myTank->getPosition()[2]/(BZDB.eval(StateDatabase::BZDB_BURROWDEPTH)-0.1f)) * ((float)viewHeight/2.0f));
   }
 
+  // draw shot reload status
+  if (BZDB.isTrue("displayReloadTimer")) {
+    renderShots(myTank);
+  }
 
   // draw cracks
   if (showCracks)
@@ -1361,7 +1353,8 @@ void			HUDRenderer::renderPlaying(SceneRenderer& renderer)
   renderAlerts();
 
   // show player scoreboard
-  if (BZDB.isTrue("displayScore")) renderScoreboard();
+  scoreboard->setRoaming(roaming);
+  scoreboard->render(false);
 
   // draw flag help
   if (flagHelpClock.isOn()) {
@@ -1380,10 +1373,11 @@ void			HUDRenderer::renderPlaying(SceneRenderer& renderer)
   }
 
   if (myTank && globalClock.isOn()) {
-    float y = 0.5f * (float)height + fm.getStrHeight(bigFontFace, bigFontSize, "0");
+    float yy = 0.5f * (float)height
+      + fm.getStrHeight(bigFontFace, bigFontSize, "0");
     if (myTank->isAutoPilot()) {
       hudColor3fv(messageColor);
-      fm.drawString(0.5f * ((float)width - autoPilotWidth), y, 0, bigFontFace,
+      fm.drawString(0.5f * ((float)width - autoPilotWidth), yy, 0, bigFontFace,
 		    bigFontSize, autoPilotLabel);
     }
   }
@@ -1404,8 +1398,6 @@ void			HUDRenderer::renderPlaying(SceneRenderer& renderer)
 
 void			HUDRenderer::renderNotPlaying(SceneRenderer& renderer)
 {
-  extern bool gameOver;
-
   // get view metrics
   const int width = window.getWidth();
   const int height = window.getHeight();
@@ -1435,7 +1427,8 @@ void			HUDRenderer::renderNotPlaying(SceneRenderer& renderer)
   renderAlerts();
 
   // show player scoreboard
-  renderScoreboard();
+  scoreboard->setRoaming(roaming);
+  scoreboard->render(true);
 
   // draw times
   renderTimes();
@@ -1447,8 +1440,7 @@ void			HUDRenderer::renderNotPlaying(SceneRenderer& renderer)
   // tell player what to do to start/resume playing
   LocalPlayer* myTank = LocalPlayer::getMyTank();
   if (myTank && globalClock.isOn()) {
-
-    float y = 0.5f * (float)height + fm.getStrHeight(bigFontFace, bigFontSize, "0");
+    float y = 0.5f * (float)viewHeight + fm.getStrHeight(bigFontFace, bigFontSize, "0");
     if (gameOver) {
       hudColor3fv(messageColor);
       fm.drawString(0.5f * ((float)width - gameOverLabelWidth), y, 0,
@@ -1474,8 +1466,6 @@ void			HUDRenderer::renderNotPlaying(SceneRenderer& renderer)
 
 void			HUDRenderer::renderRoaming(SceneRenderer& renderer)
 {
-  extern bool gameOver;
-
   // get view metrics
   const int width = window.getWidth();
   const int height = window.getHeight();
@@ -1494,11 +1484,24 @@ void			HUDRenderer::renderRoaming(SceneRenderer& renderer)
   glPushMatrix();
   glLoadIdentity();
 
+  // black out the underground if we're driving with a tank with BU
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
+  if (myTank && myTank->getPosition()[2] < 0.0f) {
+    glColor4f(0.02f, 0.01f, 0.01f, 1.0);
+    glRectf(0, 0, (float)width, (myTank->getPosition()[2]/(BZDB.eval(StateDatabase::BZDB_BURROWDEPTH)-0.1f)) * ((float)viewHeight/2.0f));
+  }
+
+  // draw shot reload status
+  if ((ROAM.getMode() == Roaming::roamViewFP) && BZDB.isTrue("displayReloadTimer")) {
+    renderShots(ROAM.getTargetTank());
+  }
+
   // draw alert messages
   renderAlerts();
 
   // show player scoreboard
-  if (BZDB.isTrue("displayScore")) renderScoreboard();
+  scoreboard->setRoaming(roaming);
+  scoreboard->render(false);
 
   // show tank labels
   if (BZDB.isTrue("displayLabels")) renderTankLabels(renderer);
@@ -1511,7 +1514,6 @@ void			HUDRenderer::renderRoaming(SceneRenderer& renderer)
     renderCompose(renderer);
 
   // display game over
-  LocalPlayer* myTank = LocalPlayer::getMyTank();
   if (myTank && globalClock.isOn()) {
     float y = 0.5f * (float)height + fm.getStrHeight(bigFontFace, bigFontSize, "0");
     if (gameOver) {
@@ -1531,132 +1533,79 @@ void			HUDRenderer::renderRoaming(SceneRenderer& renderer)
   glPopMatrix();
 }
 
-void			HUDRenderer::drawPlayerScore(const Player* player,
-			    float x1, float x2, float x3, float xs, float y)
+
+
+static int compare_float (const void* a, const void* b)
 {
-  // score
-  char score[40], kills[40];
-  if (World::getWorld()->allowRabbit())
-    sprintf(score, "%2d%% %d(%d-%d)[%d]",
-	    player->getRabbitScore(),
-	    player->getScore(),
-	    player->getWins(), player->getLosses(), player->getTeamKills());
-  else
-    sprintf(score, "%d (%d-%d)[%d]", player->getScore(),
-	    player->getWins(), player->getLosses(), player->getTeamKills());
-  if (LocalPlayer::getMyTank() != player)
-    sprintf(kills, "%d/%d", player->getLocalWins(), player->getLocalLosses());
-  else
-    strcpy(kills, "");
-
-
-  // team color
-  TeamColor teamIndex = player->getTeam();
-  if (teamIndex < RogueTeam) {
-    teamIndex = RogueTeam;
-  }
-
-  // dim the font if we're dim
-  const std::string dimString = dim ? ColorStrings[DimColor] : "";
-
-  // authentication status
-  std::string statusInfo = dimString;
-  if (BZDBCache::colorful) {
-    statusInfo += ColorStrings[CyanColor];
+  const float fa = *((const float*)a);
+  const float fb = *((const float*)b);
+  if (fa > fb) {
+    return 1;
   } else {
-    statusInfo += ColorStrings[teamIndex];;
-  }
-  if (player->isAdmin()) {
-    statusInfo += '@';
-  } else if (player->isVerified()) {
-    statusInfo += '+';
-  } else if (player->isRegistered()) {
-    statusInfo += '-';
-  } else {
-    statusInfo = ""; // don't print
-  }
-
-  std::string playerInfo = dimString;
-  // team color
-  playerInfo += ColorStrings[teamIndex];
-  // callsign
-  playerInfo += player->getCallSign();
-  // email in parenthesis
-  if (player->getEmailAddress()[0] != '\0') {
-    playerInfo += " (";
-    playerInfo += player->getEmailAddress();
-    playerInfo += ")";
-  }
-  // carried flag
-  bool coloredFlag = false;
-  FlagType* flagd = player->getFlag();
-  if (flagd != Flags::Null) {
-    // color special flags
-    if (BZDBCache::colorful) {
-      if ((flagd == Flags::ShockWave)   ||
-	  (flagd == Flags::Genocide)    ||
-	  (flagd == Flags::Laser)       ||
-	  (flagd == Flags::GuidedMissile)) {
-	playerInfo += ColorStrings[WhiteColor];
-      } else if (flagd->flagTeam != NoTeam) {
-	// use team color for team flags
-	playerInfo += ColorStrings[flagd->flagTeam];
-      }
-      coloredFlag = true;
-    }
-    playerInfo += "/";
-    playerInfo += (flagd->endurance == FlagNormal ? flagd->flagName : flagd->flagAbbv);
-    // back to original color
-    if (coloredFlag) {
-      playerInfo += ColorStrings[teamIndex];
-    }
-  }
-  // status
-  if (player->isPaused())
-    playerInfo += "[p]";
-  else if (player->isNotResponding())
-    playerInfo += "[nr]";
-  else if (player->isAutoPilot())
-    playerInfo += "[auto]";
-
-  FontManager &fm = FontManager::instance();
-  fm.setDimFactor(dimFactor);
-
-  // draw
-  if (player->getTeam() != ObserverTeam) {
-    hudColor3fv(Team::getRadarColor(teamIndex));
-    fm.drawString(x1, y, 0, minorFontFace, minorFontSize, score);
-    fm.drawString(x2, y, 0, minorFontFace, minorFontSize, kills);
-  }
-  fm.drawString(x3, y, 0, minorFontFace, minorFontSize, playerInfo);
-  if (statusInfo.size() > 0) {
-    fm.drawString(xs, y, 0, minorFontFace, minorFontSize, statusInfo);
-  }
-  if (BZDB.isTrue("debugHud")) {
-    printf ("playerInfo: %s\n", playerInfo.c_str()); //FIXME
-  }
-
-  // draw hunting status
-  const float x4 = xs - huntArrowWidth;
-  const float x5 = xs - huntedArrowWidth;
-  if (player->isHunted()) {
-    fm.drawString(x5, y, 0, minorFontFace, minorFontSize, "Hunt->");
-  } else if (getHuntIndicator()) {
-    fm.drawString(x4, y, 0, minorFontFace, minorFontSize, "->");
+    return -1;
   }
 }
 
-void			HUDRenderer::drawTeamScore(int teamIndex, float x1, float y)
+void			HUDRenderer::renderShots(const Player* target)
 {
-  char score[44];
-  Team& team = World::getWorld()->getTeam(teamIndex);
-  sprintf(score, "%d (%d-%d) %d", team.won - team.lost, team.won, team.lost, team.size);
+  // get the target tank
+  if (!target) return;
 
-  FontManager &fm = FontManager::instance();
-  hudColor3fv(Team::getRadarColor((TeamColor)teamIndex));
-  fm.drawString(x1, y, 0, minorFontFace, minorFontSize, score);
+  // get view metrics
+  const int width = window.getWidth();
+  const int height = window.getHeight();
+  const int viewHeight = window.getViewHeight();
+  const int centerx = width >> 1;
+  const int centery = viewHeight >> 1;
+
+  const int indicatorWidth = width / 50;
+  const int indicatorHeight = height / 80;
+  const int indicatorSpace = indicatorHeight / 10 + 2;
+  const int indicatorLeft = centerx + maxMotionSize + indicatorWidth + 16;
+  const int indicatorTop = centery - (int)(0.5f * (indicatorHeight + indicatorSpace) * target->getMaxShots());
+
+  const int maxShots = target->getMaxShots();
+
+  float* factors = new float[maxShots];
+
+  // tally the reload values
+  for (int i = 0; i < maxShots; ++i) {
+    const ShotPath* shot = target->getShot(i);
+    factors[i] = 1.0f;
+    if (shot) {
+      const TimeKeeper currentTime = shot->getCurrentTime();
+      const TimeKeeper startTime = shot->getStartTime();
+      const float reloadTime = shot->getReloadTime();
+      factors[i] = float(1 - ((reloadTime - (currentTime - startTime)) / reloadTime));
+      if (factors[i] > 1.0f) factors[i] = 1.0f;
+    }
+  }
+
+  // sort the reload values
+  qsort(factors, maxShots, sizeof(float), compare_float);
+
+  // draw the reload values
+  glEnable(GL_BLEND);
+  for (int i = 0; i < maxShots; ++i) {
+    const int myWidth = int(indicatorWidth * factors[i]);
+    const int myTop = indicatorTop + i * (indicatorHeight + indicatorSpace);
+    if (factors[i] < 1.0f) {
+      hudColor4f(0.0f, 1.0f, 0.0f, 0.5f); // green
+      glRecti(indicatorLeft, myTop, indicatorLeft + myWidth, myTop + indicatorHeight);
+      hudColor4f(1.0f, 0.0f, 0.0f, 0.5f); // red
+      glRecti(indicatorLeft + myWidth + 1, myTop, indicatorLeft + indicatorWidth,
+	      myTop + indicatorHeight);
+    } else {
+      hudColor4f(1.0f, 1.0f, 1.0f, 0.5f); // white
+      glRecti(indicatorLeft, myTop, indicatorLeft + myWidth, myTop + indicatorHeight);
+    }
+  }
+  glDisable(GL_BLEND);
+
+  delete[] factors;
 }
 
+
 // Local Variables: ***
 // mode:C++ ***
 // tab-width: 8 ***
@@ -1664,4 +1613,3 @@ void			HUDRenderer::drawTeamScore(int teamIndex, float x1, float y)
 // indent-tabs-mode: t ***
 // End: ***
 // ex: shiftwidth=2 tabstop=8
-
