@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2007 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __INPUTMENU_H__
@@ -40,9 +40,14 @@ public:
   void execute();
   void resize(int width, int height);
   static void callback(HUDuiControl* w, void* data);
+
+  void fillJSOptions();
+
 private:
   HUDuiControl*	   keyMapping;
   HUDuiList*	   activeInput;
+  HUDuiList*	   jsx;
+  HUDuiList*	   jsy;
   KeyboardMapMenu* keyboardMapMenu;
 };
 
