@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2007 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,27 +7,18 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-// bzflag common header
-#include "common.h"
-
-// system headers
-#include <stdio.h>
-#include <string>
-#include <vector>
+// interface header
+#include "SceneBuilder.h"
 
 // local implemenation headers
-#include "SceneBuilder.h"
-#include "SceneRenderer.h"
-#include "SceneDatabase.h"
 #include "ZSceneDatabase.h"
 #include "BSPSceneDatabase.h"
 #include "World.h"
 
 // scene node implemenation headers
-#include "WallSceneNode.h"
 #include "MeshPolySceneNode.h"
 #include "TankSceneNode.h"
 #include "BoxSceneNodeGenerator.h"
@@ -44,10 +35,10 @@
 
 // common implementation headers
 #include "StateDatabase.h"
+#include "SceneRenderer.h"
 #include "BZDBCache.h"
 #include "TextureManager.h"
-#include "BzMaterial.h"
-#include "DynamicColor.h"
+#include "ObstacleList.h"
 #include "ObstacleMgr.h"
 
 
@@ -206,11 +197,8 @@ SceneDatabaseBuilder::~SceneDatabaseBuilder()
 SceneDatabase* SceneDatabaseBuilder::make(const World* world)
 {
   // set LOD flags
-  wallLOD = BZDBCache::lighting && BZDBCache::zbuffer;
-  baseLOD = BZDBCache::lighting && BZDBCache::zbuffer;
-  boxLOD = BZDBCache::lighting && BZDBCache::zbuffer;
-  pyramidLOD = BZDBCache::lighting && BZDBCache::zbuffer;
-  teleporterLOD = BZDBCache::lighting && BZDBCache::zbuffer;
+  const bool doLODs = BZDBCache::lighting && BZDBCache::zbuffer;
+  wallLOD = baseLOD = boxLOD = pyramidLOD = teleporterLOD = doLODs;
 
   // pick type of database
   SceneDatabase* db;
@@ -228,7 +216,6 @@ SceneDatabase* SceneDatabaseBuilder::make(const World* world)
   world->freeInsideNodes();
 
 
-
   // add nodes to database
   unsigned int i;
 
@@ -265,6 +252,8 @@ SceneDatabase* SceneDatabaseBuilder::make(const World* world)
   // add the water level node
   addWaterLevel(db, world);
 
+  db->finalizeStatics();
+
   return db;
 }
 
@@ -292,11 +281,15 @@ void SceneDatabaseBuilder::addWaterLevel(SceneDatabase* db,
   t[0][0] = t[0][1] = t[1][1] = t[3][0] = 0.0f;
   t[1][0] = t[2][0] = t[2][1] = t[3][1] = 2.0f;
 
+  // get the material
+  const BzMaterial* mat = world->getWaterMaterial();
+  const bool noRadar = mat->getNoRadar();
+  const bool noShadow = mat->getNoShadow();
+
   MeshPolySceneNode* node =
-    new MeshPolySceneNode(plane, v, n, t);
+    new MeshPolySceneNode(plane, noRadar, noShadow, v, n, t);
 
   // setup the material
-  const BzMaterial* mat = world->getWaterMaterial();
   MeshSceneNodeGenerator::setupNodeMaterial(node, mat);
 
   db->addStaticNode(node, false);
@@ -307,6 +300,10 @@ void SceneDatabaseBuilder::addWaterLevel(SceneDatabase* db,
 
 void SceneDatabaseBuilder::addWall(SceneDatabase* db, const WallObstacle& o)
 {
+  if (o.getHeight() <= 0.0f) {
+    return;
+  }
+
   int part = 0;
   WallSceneNode* node;
   ObstacleSceneNodeGenerator* nodeGen = new WallSceneNodeGenerator (&o);
