@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2007 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
@@ -16,8 +16,6 @@
 /* system implementation headers */
 #include <fstream>
 #include <time.h>
-#include <vector>
-#include <string>
 #include <assert.h>
 
 /* common implementation headers */
@@ -27,12 +25,15 @@
 #include "CollisionManager.h"
 #include "DynamicColor.h"
 #include "TextureMatrix.h"
-#include "BzMaterial.h"
 #include "PhysicsDriver.h"
-#include "MeshTransform.h"
 #include "FlagSceneNode.h"
 #include "ObstacleMgr.h"
-
+#include "MeshDrawInfo.h"
+#include "MeshDrawMgr.h"
+#include "DirectoryNames.h"
+#include "GameTime.h"
+#include "WallObstacle.h"
+#include "MeshObstacle.h"
 
 //
 // World
@@ -61,6 +62,8 @@ World::World() :
   waterLevel = -1.0f;
   waterMaterial = NULL;
   linkMaterial = NULL;
+  drawInfoCount = 0;
+  drawInfoArray = NULL;
 }
 
 
@@ -69,6 +72,7 @@ World::~World()
   int i;
   freeFlags();
   freeInsideNodes();
+  freeMeshDrawMgrs();
   for (i = 0; i < curMaxPlayers; i++)
     delete players[i];
   delete[] players;
@@ -523,6 +527,53 @@ void			World::freeFlags()
   flagWarpNodes = NULL;
 }
 
+void			World::makeMeshDrawMgrs()
+{
+  // make the display list managers for source meshes
+  std::vector<MeshObstacle*> sourceMeshes;
+  OBSTACLEMGR.getSourceMeshes(sourceMeshes);
+  unsigned int count = sourceMeshes.size();
+  drawInfoArray = new MeshDrawInfo*[count];
+  drawInfoCount = 0;
+  for (unsigned int i = 0; i < count; i++) {
+    MeshDrawInfo* di = (MeshDrawInfo*) sourceMeshes[i]->getDrawInfo();
+    if ((di != NULL) && !di->isCopy()) {
+      MeshDrawMgr* dm = new MeshDrawMgr(di);
+      di->setDrawMgr(dm);
+      drawInfoArray[drawInfoCount] = di;
+      drawInfoCount++;
+    }
+  }
+  return;
+}
+
+
+void			World::freeMeshDrawMgrs()
+{
+  for (int i = 0; i < drawInfoCount; i++) {
+    MeshDrawInfo* di = drawInfoArray[i];
+    MeshDrawMgr* dm = di->getDrawMgr();
+    delete dm;
+    di->setDrawMgr(NULL);
+  }
+  drawInfoCount = 0;
+  delete[] drawInfoArray;
+  drawInfoArray = NULL;
+  return;
+}
+
+
+void			World::updateAnimations(float /*dt*/)
+{
+  const double gameTime = GameTime::getStepTime();
+  for (int i = 0; i < drawInfoCount; i++) {
+    MeshDrawInfo* di = drawInfoArray[i];
+    di->updateAnimation(gameTime);
+  }
+  return;
+}
+
+
 void			World::freeInsideNodes() const
 {
   unsigned int i;
@@ -575,9 +626,9 @@ void		World::makeLinkMaterial()
   int dyncolId = DYNCOLORMGR.findColor(name);
   if (dyncolId < 0) {
     DynamicColor* dyncol = new DynamicColor;
-    dyncol->setLimits(0, 0.0f, 0.1f); // red
-    dyncol->setLimits(1, 0.0f, 0.1f); // green
-    dyncol->setLimits(2, 0.0f, 0.1f); // blue
+    dyncol->setLimits(0, 0.0f, 0.25f); // red
+    dyncol->setLimits(1, 0.0f, 0.25f); // green
+    dyncol->setLimits(2, 0.0f, 0.25f); // blue
     dyncol->setLimits(3, 0.75f, 0.75f); // alpha
     // period, offset, weight
     float params[3] = {2.0f, 0.0f, 1.0f};
@@ -607,6 +658,7 @@ void		World::makeLinkMaterial()
   mat.setDynamicColor(dyncolId);
   mat.setTexture("telelink");
   mat.setTextureMatrix(texmatId);
+  mat.setNoLighting(true);
   mat.setName(name);
   linkMaterial = MATERIALMGR.addMaterial(&mat);
 
@@ -633,6 +685,28 @@ void			World::initFlag(int index)
   }
 }
 
+void			World::updateWind(float /*dt*/)
+{
+  const float minWindSpeed = 0.0f; // FIXME - BZDB
+  const float maxWindSpeed = 10.0f; // FIXME - BZDB
+
+  // pretty cheezy, should be fields and such
+  const double gt = GameTime::getStepTime();
+
+  const double oneMinuteFactor = (1.0 / (60.0 * (M_PI * 2.0)));
+  const float wsf = (float)(0.5 + (0.5 * cos(gt * 15.0f * oneMinuteFactor)));
+  const float windSpeed = ((1.0f - wsf) * minWindSpeed) +
+			  (wsf * maxWindSpeed);
+
+  const float windAngle = (float)((M_PI * 2.0) *
+				  (cos(gt * 3.0f * oneMinuteFactor) +
+				   cos(gt * 10.0f * oneMinuteFactor)));
+
+  wind[0] = windSpeed * cosf(windAngle);
+  wind[1] = windSpeed * sinf(windAngle);
+  wind[2] = 0.0f;
+}
+
 void			World::updateFlag(int index, float dt)
 {
   if (!flagNodes) return;
@@ -754,18 +828,40 @@ void			World::updateFlag(int index, float dt)
   // move flag scene node
   flagNodes[index]->move(flags[index].position);
 
-  // narrow flag on tank turns with tank (so it's almost invisible head-on)
-  if (flag.type == Flags::Narrow && flag.status == FlagOnTank) {
-    for (int i = 0; i < curMaxPlayers; i++)
-      if (players[i] && players[i]->getId() == flag.owner) {
-	const float* dir = players[i]->getForward();
-	flagNodes[index]->setBillboard(false);
-	flagNodes[index]->turn(atan2f(dir[1], dir[0]));
-	break;
-      }
+  // setup the flag angle
+  if (flag.status != FlagOnTank) {
+    flagNodes[index]->setWind(wind, dt);
+    flagNodes[index]->setBillboard(true);
   }
   else {
-    flagNodes[index]->setBillboard(true);
+    const Player* flagPlayer = NULL;
+    for (int i = 0; i < curMaxPlayers; i++) {
+      const Player* p = players[i];
+      if (p && p->getId() == flag.owner) {
+	flagPlayer = p;
+	break;
+      }
+    }
+    if (flagPlayer != NULL) {
+      if (flag.type == Flags::Narrow) {
+	flagNodes[index]->setAngle(flagPlayer->getAngle());
+	flagNodes[index]->setBillboard(false);
+      } else {
+	float myWind[3];
+	getWind(myWind, flagPlayer->getPosition());
+	const float* vel = flagPlayer->getVelocity();
+	myWind[0] -= vel[0];
+	myWind[1] -= vel[1];
+	if (flagPlayer->isFalling()) {
+	  myWind[2] -= vel[2];
+	}
+	flagNodes[index]->setWind(myWind, dt);
+	flagNodes[index]->setBillboard(true);
+      }
+    } else {
+      flagNodes[index]->setWind(wind, dt); // assumes homogeneous wind
+      flagNodes[index]->setBillboard(true);
+    }
   }
 }
 
@@ -806,9 +902,70 @@ void			World::addFlags(SceneDatabase* scene, bool seerView)
 }
 
 
+static std::string indent = "";
+
+
+static void writeDefaultOBJMaterials(std::ostream& out)
+{
+  typedef struct {
+    const char* name;
+    const char* texture;
+    float color[4];
+  } MatProps;
+  const MatProps defaultMats[] = {
+    {"std_ground",	"std_ground.png",	{0.5f, 0.5f, 0.5f, 1.0f}},
+    {"boxtop",		"roof.png",		{1.0f, 1.0f, 0.9f, 1.0f}},
+    {"boxwall",		"boxwall.png",		{1.0f, 0.9f, 0.8f, 1.0f}},
+    {"pyrwall",		"pyrwall.png",		{0.8f, 0.8f, 1.0f, 1.0f}},
+    {"telefront",	"telelink.png",		{1.0f, 0.0f, 0.0f, 0.5f}},
+    {"teleback",	"telelink.png",		{0.0f, 1.0f, 0.0f, 0.5f}},
+    {"telerim",		"caution.png",		{1.0f, 1.0f, 0.0f, 0.5f}},
+    {"basetop_team1",	"red_basetop.png",	{1.0f, 0.8f, 0.8f, 1.0f}},
+    {"basewall_team1",	"red_basewall.png",	{1.0f, 0.8f, 0.8f, 1.0f}},
+    {"basetop_team2",	"green_basetop.png",	{0.8f, 1.0f, 0.8f, 1.0f}},
+    {"basewall_team2",	"green_basewall.png",	{0.8f, 1.0f, 0.8f, 1.0f}},
+    {"basetop_team3",	"blue_basetop.png",	{0.8f, 0.8f, 1.0f, 1.0f}},
+    {"basewall_team3",	"blue_basewall.png",	{0.8f, 0.8f, 1.0f, 1.0f}},
+    {"basetop_team4",	"purple_basetop.png",	{1.0f, 0.8f, 1.0f, 1.0f}},
+    {"basewall_team4",	"purple_basewall.png",	{1.0f, 0.8f, 1.0f, 1.0f}}
+  };
+  const int count = sizeof(defaultMats) / sizeof(defaultMats[0]);
+  BzMaterial mat;
+  for (int i = 0; i < count; i++) {
+    const MatProps& mp = defaultMats[i];
+    mat.setName(mp.name);
+    mat.setTexture(mp.texture);
+    mat.setDiffuse(mp.color);
+    mat.printMTL(out, "");
+  }
+  return;
+}
+
+
+static void writeOBJGround(std::ostream& out)
+{
+  const float ws = BZDBCache::worldSize / 2.0f;
+  const float ts = BZDBCache::worldSize / 100.0f;
+  out << "o bzground" << std::endl;
+  out << "v " << -ws << " " << -ws << " 0" << std::endl;
+  out << "v " << +ws << " " << -ws << " 0" << std::endl;
+  out << "v " << +ws << " " << +ws << " 0" << std::endl;
+  out << "v " << -ws << " " << +ws << " 0" << std::endl;
+  out << "vt " << -ts << " " << -ts << std::endl;
+  out << "vt " << +ts << " " << -ts << std::endl;
+  out << "vt " << +ts << " " << +ts << std::endl;
+  out << "vt " << -ts << " " << +ts << std::endl;
+  out << "vn 0 0 1" << std::endl;
+  out << "usemtl std_ground" << std::endl;
+  out << "f -4/-4/-1 -3/-3/-1 -2/-2/-1 -1/-1/-1" << std::endl;
+  out << std::endl;
+  return;
+}
+
+
 static void writeBZDBvar (const std::string& name, void *userData)
 {
-  std::ofstream *out = (std::ofstream *)userData;
+  std::ofstream& out = *((std::ofstream*)userData);
   if ((BZDB.getPermission(name) == StateDatabase::Server)
       && (BZDB.get(name) != BZDB.getDefault(name))
       && (name != "poll")) {
@@ -816,16 +973,35 @@ static void writeBZDBvar (const std::string& name, void *userData)
     if (BZDB.get(name).find(' ') != std::string::npos) {
       qmark = '"';
     }
-    (*out) << "  -set " << name << " "
-			<< qmark << BZDB.get(name) << qmark << std::endl;
+    out << indent << "  -set " << name << " "
+		  << qmark << BZDB.get(name) << qmark << std::endl;
   }
   return;
 }
 
 
-bool			World::writeWorld(std::string filename)
+bool World::writeWorld(const std::string& filename, std::string& fullname)
 {
-  std::ostream *stream = FILEMGR.createDataOutStream(filename.c_str());
+  const bool saveAsOBJ = BZDB.isTrue("saveAsOBJ");
+  if (saveAsOBJ) {
+    indent = "# ";
+  } else {
+    indent = "";
+  }
+
+  fullname = getWorldDirName();
+  fullname += filename;
+  if (saveAsOBJ) {
+    if (strstr(fullname.c_str(), ".obj") == NULL) {
+      fullname += ".obj";
+    }
+  } else {
+    if (strstr(fullname.c_str(), ".bzw") == NULL) {
+      fullname += ".bzw";
+    }
+  }
+
+  std::ostream *stream = FILEMGR.createDataOutStream(fullname.c_str());
   if (stream == NULL) {
     return false;
   }
@@ -838,14 +1014,14 @@ bool			World::writeWorld(std::string filename)
 
   // Write the Server Options
   {
-    out << "options" << std::endl;
+    out << indent << "options" << std::endl;
 
     // FIXME - would be nice to get a few other thing
     //	 -fb, -sb, rabbit style, a real -mp, etc... (also, flags?)
 
     if (allowTeamFlags()) {
-      out << "  -c" << std::endl;
-      out << "  -mp 2,";
+      out << indent << "  -c" << std::endl;
+      out << indent << "  -mp 2,";
       for (int t = RedTeam; t <= PurpleTeam; t++) {
 	if (getBase(t, 0) != NULL)
 	  out << "2,";
@@ -855,29 +1031,30 @@ bool			World::writeWorld(std::string filename)
       out << "2" << std::endl;
     }
     if (allowRabbit())
-      out << "  -rabbit" << std::endl;
+      out << indent << "  -rabbit" << std::endl;
     if (allowJumping())
-      out << "  -j" << std::endl;
+      out << indent << "  -j" << std::endl;
     if (allShotsRicochet())
-      out << "  +r" << std::endl;
+      out << indent << "  +r" << std::endl;
     if (allowHandicap())
-      out << "  -handicap" << std::endl;
-    if (allowInertia()) {
-      out << "  -a " << getLinearAcceleration() << " "
+      out << indent << "  -handicap" << std::endl;
+    if (allowAntidote()) {
+      out << indent << "  -sa" << std::endl;
+      out << indent << "  -st " << getFlagShakeTimeout() << std::endl;
+      out << indent << "  -sw " << getFlagShakeWins() << std::endl;
+    }
+    if ((getLinearAcceleration() != 0.0f) ||
+	(getAngularAcceleration() != 0.0f)) {
+      out << indent << "  -a " << getLinearAcceleration() << " "
 		     << getAngularAcceleration() << std::endl;
     }
-    if (allowAntidote()) {
-      out << "  -sa" << std::endl;
-      out << "  -st " << getFlagShakeTimeout() << std::endl;
-      out << "  -sw " << getFlagShakeWins() << std::endl;
-    }
 
-    out << "  -ms " << getMaxShots() << std::endl;
+    out << indent << "  -ms " << getMaxShots() << std::endl;
 
     // Write BZDB server variables that aren't defaults
     BZDB.iterate (writeBZDBvar, &out);
 
-    out << "end" << std::endl << std::endl;
+    out << indent << "end" << std::endl << std::endl;
   }
 
   // Write World object
@@ -887,52 +1064,70 @@ bool			World::writeWorld(std::string filename)
     if ((worldSize != atof(BZDB.getDefault(StateDatabase::BZDB_WORLDSIZE).c_str()))
     ||  (flagHeight != atof(BZDB.getDefault(StateDatabase::BZDB_FLAGHEIGHT).c_str())))
     {
-      out << "world" << std::endl;
+      out << indent << "world" << std::endl;
       if (worldSize != atof(BZDB.getDefault(StateDatabase::BZDB_WORLDSIZE).c_str())) {
-	out << "  size " << worldSize / 2.0f << std::endl;
+	out << indent << "  size " << worldSize / 2.0f << std::endl;
       }
       if (flagHeight != atof(BZDB.getDefault(StateDatabase::BZDB_FLAGHEIGHT).c_str())) {
-	out << "  flagHeight " << flagHeight << std::endl;
+	out << indent << "  flagHeight " << flagHeight << std::endl;
       }
-      out << "end" << std::endl << std::endl;
+      out << indent << "end" << std::endl << std::endl;
     }
   }
 
   // Write dynamic colors
-  DYNCOLORMGR.print(out, "");
+  DYNCOLORMGR.print(out, indent);
 
   // Write texture matrices
-  TEXMATRIXMGR.print(out, "");
+  TEXMATRIXMGR.print(out, indent);
 
   // Write materials
-  MATERIALMGR.print(out, "");
+  if (!saveAsOBJ) {
+    MATERIALMGR.print(out, indent);
+  } else {
+    const std::string mtlname = filename + ".mtl";
+    const std::string mtlfile = getWorldDirName() + mtlname;
+    std::ostream* mtlStream = FILEMGR.createDataOutStream(mtlfile.c_str());
+    if (mtlStream != NULL) {
+      out << "mtllib " << mtlname << std::endl; // index the mtl file
+      out << std::endl;
+      *mtlStream << "# BZFlag client: saved world on " << ctime(&nowTime);
+      *mtlStream << std::endl;
+      writeDefaultOBJMaterials(*mtlStream);
+      MATERIALMGR.printMTL(*mtlStream, "");
+      delete mtlStream;
+    }
+  }
 
   // Write physics drivers
-  PHYDRVMGR.print(out, "");
+  PHYDRVMGR.print(out, indent);
 
   // Write obstacle transforms
-  TRANSFORMMGR.print(out, "");
+  TRANSFORMMGR.print(out, indent);
 
   // Write water level
   {
     if (waterLevel >= 0.0f) {
-      out << "waterLevel" << std::endl;
-      out << "  height " << waterLevel << std::endl;
-      out << "  matref ";
+      out << indent << "waterLevel" << std::endl;
+      out << indent << "  height " << waterLevel << std::endl;
+      out << indent << "  matref ";
       MATERIALMGR.printReference(out, waterMaterial);
       out << std::endl;
-      out << "end" << std::endl << std::endl;
+      out << indent << "end" << std::endl << std::endl;
     }
   }
 
   // Write the world obstacles
   {
-    OBSTACLEMGR.print(out, "");
+    if (saveAsOBJ) {
+      writeOBJGround(out);
+    }
+    OBSTACLEMGR.print(out, indent);
   }
 
   // Write links
   {
-    links.print(out, "");
+    links.print(out, indent);
   }
 
   // Write weapons
@@ -940,23 +1135,23 @@ bool			World::writeWorld(std::string filename)
     for (std::vector<Weapon>::iterator it = weapons.begin();
 	 it != weapons.end(); ++it) {
       Weapon weapon = *it;
-      out << "weapon" << std::endl;
+      out << indent << "weapon" << std::endl;
       if (weapon.type != Flags::Null) {
-	out << "\ttype " << weapon.type->flagAbbv << std::endl;
+	out << indent << "  type " << weapon.type->flagAbbv << std::endl;
       }
-      out << "\tposition " << weapon.pos[0] << " " << weapon.pos[1] << " "
+      out << indent << "  position " << weapon.pos[0] << " " << weapon.pos[1] << " "
 			   << weapon.pos[2] << std::endl;
-      out << "\trotation " << ((weapon.dir * 180.0) / M_PI) << std::endl;
-      out << "\tinitdelay " << weapon.initDelay << std::endl;
+      out << indent << "  rotation " << ((weapon.dir * 180.0) / M_PI) << std::endl;
+      out << indent << "  initdelay " << weapon.initDelay << std::endl;
       if (weapon.delay.size() > 0) {
-	out << "\tdelay";
+	out << indent << "  delay";
 	for (std::vector<float>::iterator dit = weapon.delay.begin();
 	     dit != weapon.delay.end(); ++dit) {
 	  out << " " << (float)*dit;
 	}
 	out << std::endl;
       }
-      out << "end" << std::endl << std::endl;
+      out << indent << "end" << std::endl << std::endl;
     }
   }
 
@@ -965,14 +1160,14 @@ bool			World::writeWorld(std::string filename)
     for (std::vector<EntryZone>::iterator it = entryZones.begin();
 	 it != entryZones.end(); ++it) {
       EntryZone zone = *it;
-      out << "zone" << std::endl;
-      out << "\tposition " << zone.pos[0] << " " << zone.pos[1] << " "
+      out << indent << "zone" << std::endl;
+      out << indent << "  position " << zone.pos[0] << " " << zone.pos[1] << " "
 			   << zone.pos[2] << std::endl;
-      out << "\tsize " << zone.size[0] << " " << zone.size[1] << " "
+      out << indent << "  size " << zone.size[0] << " " << zone.size[1] << " "
 		       << zone.size[2] << std::endl;
-      out << "\trotation " << ((zone.rot * 180.0) / M_PI) << std::endl;
+      out << indent << "  rotation " << ((zone.rot * 180.0) / M_PI) << std::endl;
       if (zone.flags.size() > 0) {
-	out << "\tflag";
+	out << indent << "  flag";
 	std::vector<FlagType*>::iterator fit;
 	for (fit = zone.flags.begin(); fit != zone.flags.end(); ++fit) {
 	  out << " " << (*fit)->flagAbbv;
@@ -980,7 +1175,7 @@ bool			World::writeWorld(std::string filename)
 	out << std::endl;
       }
       if (zone.teams.size() > 0) {
-	out << "\tteam";
+	out << indent << "  team";
 	std::vector<TeamColor>::iterator tit;
 	for (tit = zone.teams.begin(); tit != zone.teams.end(); ++tit) {
 	  out << " " << (*tit);
@@ -988,14 +1183,14 @@ bool			World::writeWorld(std::string filename)
 	out << std::endl;
       }
       if (zone.safety.size() > 0) {
-	out << "\tsafety";
+	out << indent << "  safety";
 	std::vector<TeamColor>::iterator sit;
 	for (sit = zone.safety.begin(); sit != zone.safety.end(); ++sit) {
 	  out << " " << (*sit);
 	}
 	out << std::endl;
       }
-      out << "end" << std::endl << std::endl;
+      out << indent << "end" << std::endl << std::endl;
     }
   }
 
@@ -1037,7 +1232,7 @@ void World::drawCollisionGrid() const
 
   glGetBooleanv (GL_TEXTURE_2D, &usingTextures);
   glDisable (GL_TEXTURE_2D);
-  
+
   COLLISIONMGR.draw (&drawLines);
 
   if (usingTextures) {
@@ -1047,6 +1242,20 @@ void World::drawCollisionGrid() const
   return;
 }
 
+RemotePlayer* World::getCurrentRabbit() const
+{
+  if (players == NULL) {
+    return NULL;
+  }
+  for (int i = 0; i < curMaxPlayers; i++) {
+    RemotePlayer* p = players[i];
+    if (p && p->isAlive() && (p->getTeam() == RabbitTeam)) {
+      return p;
+    }
+  }
+  return NULL;
+}
+
 
 // Local Variables: ***
 // mode: C++ ***
