@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2007 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,28 +7,34 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __CUSTOMBASE_H__
 #define __CUSTOMBASE_H__
 
-// system headers
+#include "common.h"
+
+/* interface header */
+#include "WorldFileObstacle.h"
+
+/* system interface headers */
 #include <iostream>
 
-// bzfs-specific headers
-#include "WorldFileObstacle.h"
+/* local interface headers */
 #include "WorldInfo.h"
 
 
 class CustomBase : public WorldFileObstacle {
   public:
     CustomBase();
-  virtual bool read(const char *cmd, std::istream&);
-    virtual void write(WorldInfo*) const;
+    virtual bool read(const char *cmd, std::istream&);
+    virtual void writeToGroupDef(GroupDefinition*) const;
 
   protected:
     int color;
+    bool triggerWorldWep;
+    std::string worldWepType;
 };
 
 #endif  /* __CUSTOMBASE_H__ */
