@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2007 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __CUSTOMWORLD_H__
@@ -37,6 +37,11 @@ class CustomWorld : public WorldFileObject {
     double _fHeight;
 };
 
+
+extern std::map<std::string,bz_CustomMapObjectHandler*>	customObjectMap;
+
+void registerCustomMapObject ( const char* object, bz_CustomMapObjectHandler *handler );
+void removeCustomMapObject ( const char* object );
 #endif  /* __CUSTOMWORLD_H__ */
 
 // Local variables: ***
