@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993 - 2007 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -49,6 +49,8 @@ ListServerLink::ListServerLink(std::string listServerURL,
   setDNSCachingTime(-1);
   setTimeout(10);
 
+  publiclyDisconected = false;
+
   if (clOptions->pingInterface != "")
     setInterface(clOptions->pingInterface);
 
@@ -94,6 +96,8 @@ ListServerLink::~ListServerLink()
 
 void ListServerLink::finalization(char *data, unsigned int length, bool good)
 {
+	publiclyDisconected = !good;
+
   queuedRequest = false;
   if (good && (length < 2048)) {
     char buf[2048];
@@ -101,10 +105,10 @@ void ListServerLink::finalization(char *data, unsigned int length, bool good)
     int bytes = length;
     buf[bytes]=0;
     char* base = buf;
-    static char *tokGoodIdentifier = "TOKGOOD: ";
-    static char *tokBadIdentifier = "TOKBAD: ";
-    static char *unknownPlayer = "UNK: ";
-    static char *bzIdentifier = "BZID: ";
+    const char tokGoodIdentifier[] = "TOKGOOD: ";
+    const char tokBadIdentifier[]  = "TOKBAD: ";
+    const char unknownPlayer[]     = "UNK: ";
+    const char bzIdentifier[]      = "BZID: ";
     // walks entire reply including HTTP headers
     while (*base) {
       // find next newline
@@ -113,7 +117,7 @@ void ListServerLink::finalization(char *data, unsigned int length, bool good)
       // if no newline then no more complete replies
       if (*scan != '\r' && *scan != '\n') break;
       while (*scan && (*scan == '\r' || *scan == '\n')) *scan++ = '\0';
-      DEBUG4("Got line: \"%s\"\n", base);
+      logDebugMessage(4,"Got line: \"%s\"\n", base);
       // TODO don't do this if we don't want central logins
 
       // is player globally registered ?
@@ -140,17 +144,17 @@ void ListServerLink::finalization(char *data, unsigned int length, bool good)
 	std::string line = base;
 	std::vector<std::string> args = TextUtils::tokenize(line, " \t", 3, true);
 	if (args.size() < 3) {
-	  DEBUG3("Bad BZID string: %s\n", line.c_str());
+	  logDebugMessage(3,"Bad BZID string: %s\n", line.c_str());
 	} else {
 	  const std::string& bzId = args[1];
 	  const std::string& nick = args[2];
-	  DEBUG4("Got BZID: \"%s\" || \"%s\"\n", bzId.c_str(), nick.c_str());
+	  logDebugMessage(4,"Got BZID: \"%s\" || \"%s\"\n", bzId.c_str(), nick.c_str());
 	  for (int i = 0; i < curMaxPlayers; i++) {
 	    GameKeeper::Player* gkp = GameKeeper::Player::getPlayerByIndex(i);
 	    if ((gkp != NULL) &&
 		(strcasecmp(gkp->player.getCallSign(), nick.c_str()) == 0)) {
 	      gkp->setBzIdentifier(bzId);
-	      DEBUG3("Set player (%s [%i]) bzId to (%s)\n",
+	      logDebugMessage(3,"Set player (%s [%i]) bzId to (%s)\n",
 		     nick.c_str(), i, bzId.c_str());
 	      break;
 	    }
@@ -190,26 +194,29 @@ void ListServerLink::finalization(char *data, unsigned int length, bool good)
       }
 
       if (bzIdInfo == true) {
-	DEBUG3("Got BZID: %s", base);
+	logDebugMessage(3,"Got BZID: %s", base);
 	for (int i = 0; i < curMaxPlayers; i++) {
 	  GameKeeper::Player* gkp = GameKeeper::Player::getPlayerByIndex(i);
 	  if ((gkp != NULL) &&
 	      (strcasecmp(gkp->player.getCallSign(), callsign) == 0)) {
 	    gkp->setBzIdentifier(bzId);
-	    DEBUG3("Set player (%s [%i]) bzId to (%s)\n", callsign, i, bzId.c_str());
+	    logDebugMessage(3,"Set player (%s [%i]) bzId to (%s)\n", callsign, i, bzId.c_str());
 	    break;
 	  }
 	}
       }
 */
       if (authReply) {
-	DEBUG3("Got: %s", base);
-	char *group;
+	logDebugMessage(3,"Got: %s", base);
+	char *group = (char *)NULL;
+
 	// Isolate callsign from groups
 	if (verified) {
 	  group = callsign;
-	  while (*group && (*group != ':')) group++;
-	  while (*group && (*group == ':')) *group++ = 0;
+	  if (group) {
+	    while (*group && (*group != ':')) group++;
+	    while (*group && (*group == ':')) *group++ = 0;
+	  }
 	}
 	GameKeeper::Player *playerData = NULL;
 	int playerIndex;
@@ -223,7 +230,7 @@ void ListServerLink::finalization(char *data, unsigned int length, bool good)
 					 callsign))
 	    break;
 	}
-	DEBUG3("[%d]\n", playerIndex);
+	logDebugMessage(3,"[%d]\n", playerIndex);
 
 	if (playerIndex < curMaxPlayers) {
 	  if (registered) {
@@ -241,10 +248,12 @@ void ListServerLink::finalization(char *data, unsigned int length, bool good)
 	      if (verified) {
 		playerData->_LSAState = GameKeeper::Player::verified;
 		playerData->accessInfo.setPermissionRights();
-		while (*group) {
+		while (group && *group) {
 		  char *nextgroup = group;
-		  while (*nextgroup && (*nextgroup != ':')) nextgroup++;
-		  while (*nextgroup && (*nextgroup == ':')) *nextgroup++ = 0;
+		  if (nextgroup) {
+		    while (*nextgroup && (*nextgroup != ':')) nextgroup++;
+		    while (*nextgroup && (*nextgroup == ':')) *nextgroup++ = 0;
+		  }
 		  playerData->accessInfo.addGroup(group);
 		  group = nextgroup;
 		}
@@ -313,14 +322,14 @@ void ListServerLink::queueMessage(MessageType type)
   if (!queuedRequest)
     sendQueuedMessages();
   else
-    DEBUG3("There is a message already queued to the list server: not sending this one yet.\n");
+    logDebugMessage(3,"There is a message already queued to the list server: not sending this one yet.\n");
 }
 
 void ListServerLink::sendQueuedMessages()
 {
   queuedRequest = true;
   if (nextMessageType == ListServerLink::ADD) {
-    DEBUG3("Queuing ADD message to list server\n");
+    logDebugMessage(3,"Queuing ADD message to list server\n");
 
     bz_ListServerUpdateEvent	updateEvent;
     updateEvent.address = publicizeAddress;
@@ -332,7 +341,7 @@ void ListServerLink::sendQueuedMessages()
     addMe(getTeamCounts(), std::string(updateEvent.address.c_str()), std::string(updateEvent.description.c_str()), std::string(updateEvent.groups.c_str()));
     lastAddTime = TimeKeeper::getCurrent();
   } else if (nextMessageType == ListServerLink::REMOVE) {
-    DEBUG3("Queuing REMOVE message to list server\n");
+    logDebugMessage(3,"Queuing REMOVE message to list server\n");
     removeMe(publicizeAddress);
   }
   nextMessageType = ListServerLink::NONE;
