@@ -1,5 +1,5 @@
 /* bzflag
-* Copyright (c) 1993 - 2005 Tim Riker
+* Copyright (c) 1993 - 2007 Tim Riker
 *
 * This package is free software;  you can redistribute it and/or
 * modify it under the terms of the license found in the file
@@ -7,24 +7,30 @@
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
-* WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
 
 #ifndef __MASTER_BAN_LIST_H__
 #define __MASTER_BAN_LIST_H__
 
-// bzflag global header
-#include "global.h"
+// common implementation headers
+#include "cURLManager.h"
 
-class MasterBanList {
+class MasterBanList : cURLManager {
 public:
-	MasterBanList();
-	~MasterBanList();
-
 const std::string& get ( const std::string URL );
 protected:
 	std::string	data;
  private:
+	void finalization(char *cURLdata, unsigned int length, bool good);
 };
 #endif //__MASTER_BAN_LIST_H__
 
+// Local Variables: ***
+// mode:C++ ***
+// tab-width: 8 ***
+// c-basic-offset: 2 ***
+// indent-tabs-mode: t ***
+// End: ***
+// ex: shiftwidth=2 tabstop=8
+
